package com.javacodegeeks.log4j.database.example;

import java.sql.Connection;
import java.sql.Date;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.log4j.Logger;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 * HelloLog4J!
 *
 */
public class HelloLog4J
{
	private static final Logger LOG = Logger.getLogger(HelloLog4J.class);

	@Test
	public void run() {
		LOG.info("This is an example of JDBCAppender of log4j!");

		LOG.debug("Start of run()");

		try {
			// Intentionally we trigger divide by zero exception
			if(23/0 > 23) {
//				some code...
			}
		} catch(Exception e) {
			LOG.error("Execution error", e);
		}

		LOG.debug("End of run()");
	}

	@BeforeClass
	public static void setup() {
		try (Connection connection = getConnection(); Statement statement = connection.createStatement();) {
			Class.forName("org.hsqldb.jdbc.JDBCDriver");

			statement.execute("CREATE TABLE APP_LOGS(THREAD VARCHAR(20), DATE_OF_OCCURENCE DATETIME, CLASS VARCHAR(100),"
					+ "LINE_NUMBER INTEGER, LEVEL VARCHAR(10), MESSAGE VARCHAR(1000), STACKTRACE CLOB)");
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	@AfterClass
	public static void destroy() {
		try (Connection connection = getConnection(); Statement statement = connection.createStatement();) {
			ResultSet rs = statement.executeQuery("select * from APP_LOGS");
			System.out.println("Thread | Date | Class | Line Number | Level | Message | Stacktrace");
			while(rs.next()) {
				String userId = rs.getString("THREAD");
				Date date = rs.getDate("DATE_OF_OCCURENCE");
				String logger = rs.getString("CLASS");
				int line = rs.getInt("LINE_NUMBER");
				String level = rs.getString("LEVEL");
				String message = rs.getString("MESSAGE");
				String stackTrace = rs.getString("STACKTRACE");
				System.out.println(userId + " | " + date + " | " + logger + " | " + line + " | " + level + " | " 
				+ message + " | " + stackTrace);
			}
			rs.close();
			statement.executeUpdate("DROP TABLE APP_LOGS");
			connection.commit();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	public static Connection getConnection() throws SQLException {
		return DriverManager.getConnection("jdbc:hsqldb:mem:exampleDatabase", "root", "password");
	}
}
